# -*- coding: utf8 -*-

import email
import hashlib
import random
import socket
import urllib
import urllib2
import cookielib
import datetime

from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse_lazy
from django.db import models, IntegrityError
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _

from django.views.generic.base import TemplateView, RedirectView
from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from django.views.generic.edit import CreateView, UpdateView
from django.views.decorators.csrf import csrf_exempt

from jsonresponse import to_json

from .forms import SubscribeForm, NewsletterForm
from .models import Subscriber, Newsletter


class SubscribeView(CreateView):
    form_class = SubscribeForm
    model = Subscriber

    success_url = 'thanks'
subscribe = csrf_exempt(SubscribeView.as_view())


class ThanksView(TemplateView):
    template_name = 'newsletter/thanks.html'
thanks = ThanksView.as_view()


class ConfirmationView(TemplateView):
    template_name = 'newsletter/done.html'

    def get_context_data(self, token, **kwargs):
        context = super(ConfirmationView, self).get_context_data(**kwargs)
        subscriber = get_object_or_404(Subscriber, password=token, is_validated=False)
        subscriber.is_validated = True
        subscriber.save()
        return context
confirmation = ConfirmationView.as_view()


def unsubscription(request) :
    UNSUBSCRIPTION_MESSAGE = _("Enter your email for unsunscribe to the newsletter.")
    UNSUBSCRIPTION_CONFIRMATION_MESSAGE = _("You're now unsubscribed")

    if request.method == 'GET' :
        form = SubscriptionForm()
        return render(request, 'unsubscription_form.html', {'form' : form, 'message' : UNSUBSCRIPTION_MESSAGE})
    elif request.method == 'POST' :
        form = SubscriptionForm(request.POST)
        if form.is_valid() :
            cd = form.cleaned_data
            try :
                subscriber = Subscriber.objects.get(email = cd['email'])
                
                cookie_jar = cookielib.CookieJar()
                url_opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookie_jar))

                login_params = urllib.urlencode({'email' : subscriber.email, 'password' : subscriber.password})
                login_request = urllib2.Request(settings.NEWSLETTER_UNSUBSCRIPTION_URL, login_params)
                url = url_opener.open(login_request)
                
                unsubscribe_action_url = ("%s/%s" % (settings.NEWSLETTER_UNSUBSCRIPTION_URL, subscriber.email))
                unsubscribe_params = urllib.urlencode({'unsub' : 'Résilier', 'unsubconfirm' : '1'})
                unsubscribe_request = urllib2.Request(unsubscribe_action_url, unsubscribe_params)
                url = url_opener.open(unsubscribe_request)

                url.close()

                subscriber.delete()
                return render(request, 'unsubscription_form.html', {'message' : UNSUBSCRIPTION_CONFIRMATION_MESSAGE})
            except Subscriber.DoesNotExist :
                error_message = _("The entered email address does not exist. Please try another one.")
                return render(request, 'unsubscription_form.html', {'form' : form, 'message' : error_message})


class NewsletterCreateView(CreateView):
    form_class = NewsletterForm
    model = Newsletter

    success_url = reverse_lazy('newsletter-list')

    def get_form(self, *args, **kwargs):
        if not self.request.user.has_perm('newsletter.add_newsletter'):
            raise PermissionDenied()
        return super(NewsletterCreateView, self).get_form(*args, **kwargs)

    def get_initial(self):
        initial = super(NewsletterCreateView, self).get_initial()
        initial['date'] = datetime.datetime.today()
        return initial

newsletter_create = NewsletterCreateView.as_view()


class NewsletterListView(ListView):
    model = Newsletter

newsletter_list = NewsletterListView.as_view()


class NewsletterDetailView(DetailView):
    model = Newsletter

newsletter_view = NewsletterDetailView.as_view()

class NewsletterUpdateView(UpdateView):
    form_class = NewsletterForm
    model = Newsletter

    def get_form(self, *args, **kwargs):
        if not self.request.user.has_perm('newsletter.add_newsletter'):
            raise PermissionDenied()
        return super(NewsletterUpdateView, self).get_form(*args, **kwargs)

newsletter_update = NewsletterUpdateView.as_view()


class NewsletterSendView(RedirectView):
    def get_redirect_url(self, pk):
        if not self.request.user.has_perm('newsletter.add_newsletter'):
            raise PermissionDenied()

        newsletter = get_object_or_404(Newsletter, pk=pk)
        newsletter.send()
        return reverse_lazy('newsletter-list')

newsletter_send = NewsletterSendView.as_view()

@csrf_exempt
@to_json('api')
def stats(request) :
    return {'registered': Subscriber.objects.filter(is_registered=True).count()}
